//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                             CandlePatterns.mq5 |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""
#property version   "1.00"
#property indicator_chart_window
#property indicator_buffers 2       // используется 2 буфера индикатора

// ============================================================== Включения и импорт внешних модулей =============================================================+
#import "EnvironmentInfo\TimeSeries.ex5"
  int barsTotal(ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  int barShift(datetime initialBarTime, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barHighPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barLowPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barOpenPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barClosePrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  datetime barOpenTime(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
#import

//#include <HOZ_Code\Enums\Location.mqh>
//#include <HOZ_Code\Classes\GraphicalPlotting\Draw.mqh>

//input  ENUM_TIMEFRAMES i_tf = PERIOD_H1;
input bool isBearishEngulfingChecking = true;
input bool isBullishEngulfingChecking = true;
input bool toDrawPatternsInfo = true;
input int barsToCalculate = 3000;

double bearishEngulfingBuf[];               // Буфер медвежьих поглощений
double bullishEngulfingBuf[];               // Буфер бычьих поглощений
int    firstBarIndexToCalculate;

#define PREFIX "PAPATTERN_"                        // Префикс имён графических объектов
#define BULLISHENGULFING_NAME "BULLISH ENGULFING"  // Текстовое описание паттерна 'Bullish Engulfing'
#define BEARISHENGULFING_NAME "BEARISH ENGULFING"  // Текстовое описание паттерна 'Bearish Engulfing'
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                      |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
int OnInit() {
// - 1 - == Проверка корректности значений настроечных параметров индикатора ============
//  string name = WindowExpertName();
// - 2 - == Связывание буферов с индексами, определение стилей ==========================
  SetIndexBuffer(0, bearishEngulfingBuf, INDICATOR_DATA);    // Первый буфер - медвежьи поглощения
  PlotIndexSetInteger(0, PLOT_DRAW_TYPE, DRAW_NONE);         // Не отображается
  SetIndexBuffer(1, bullishEngulfingBuf, INDICATOR_DATA);    // Второй буфер - бычьи поглощения
  PlotIndexSetInteger(1, PLOT_DRAW_TYPE, DRAW_NONE);         // Не отображается
//---
  return INIT_SUCCEEDED;
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                      |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void OnDeinit(const int reason) {
  deleteAllObjects();
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Удаление всех объектов, созданных индикатором                                                                                                                 |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void deleteAllObjects() {
  ObjectsDeleteAll(0, PREFIX, 0, -1);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Расположение                                                                                                                                                  |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
enum ENUM_LOCATION {
  LOCATION_NONE,
  LOCATION_ABOVE,
  LOCATION_BELOW
};
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Отрисовка указателей на найдённые паттерны в виде стрелочек, линии хай или лоу сформированных паттернов (в зависимости от типа паттерна) и подпись этих       |
//| паттернов текстом                                                                                                                                             |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void drawPatternInfo(string patternName, int index, ENUM_LOCATION objLocation, uchar arrowCode, color clr) {
//    string arrowName = Draw :: wingdings(PREFIX + patternName + "_", objLocation, arrowCode, 2, clr, index, _Period);
//    double arrowPrice = ObjectGetDouble(0, arrowName, OBJPROP_PRICE);

    string appliedPrice;
    double price1, price2;

    if (objLocation == LOCATION_BELOW) {
      appliedPrice = "LowestPrice";
      price1 = price2 = barLowPrice(index);
//      Draw :: text(PREFIX + patternName + "_", LOCATION_BELOW, arrowPrice, clrBlack, index, _Period);
    } else {
      appliedPrice = "HighestPrice";
      price1 = price2 = barHighPrice(index);
//      Draw :: text(PREFIX + patternName + "_", LOCATION_ABOVE, arrowPrice, clrBlack, index, _Period);
    }

//    Draw :: line(PREFIX + patternName + "_", barOpenTime(index), price1, barOpenTime(index - 4), price2, STYLE_SOLID, 2, clrBlack, index);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Поиск паттерна Bullish Engulfing                                                                                                                              |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void checkingBullishEngulfing(int index) {
  if ((barClosePrice(index + 1) < barOpenPrice(index + 1)) && (barOpenPrice(index) < barLowPrice(index + 1)) && (barClosePrice(index) > barOpenPrice(index + 1))) {
    bullishEngulfingBuf[index] = barClosePrice(index + 1);
    if (toDrawPatternsInfo)
      drawPatternInfo(BULLISHENGULFING_NAME, index, LOCATION_BELOW, 241, clrBlue);
  } else {
    bullishEngulfingBuf[index] = EMPTY_VALUE;
  }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Поиск паттерна Bearish Engulfing                                                                                                                              |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void checkingBearishEngulfing(int index) {
  if ((barClosePrice(index + 1) > barOpenPrice(index + 1)) && (barOpenPrice(index) > barHighPrice(index + 1)) && (barClosePrice(index) < barOpenPrice(index + 1))) {
    bearishEngulfingBuf[index] = barClosePrice(index + 1);
    Print("bearishEngulfingBuf[", index, "] = ", bearishEngulfingBuf[index]);
    if (toDrawPatternsInfo)
      drawPatternInfo(BEARISHENGULFING_NAME, index, LOCATION_ABOVE, 242, clrRed);
  } else {
    bearishEngulfingBuf[index] = EMPTY_VALUE;
  }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Расчет значений индикатора                                                                                                                                    |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void checkingPatterns(int limit) {
  for (int i = limit; i > 0; i--) {
    if (isBearishEngulfingChecking) checkingBearishEngulfing(i);
    if (isBullishEngulfingChecking) checkingBullishEngulfing(i);
  }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Определение индекса бара, с которого необходимо производить перерасчет                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
int recalcIndex(int calculatedYet) {
  firstBarIndexToCalculate = barsTotal() - 1;
  
  if (barsToCalculate > 0 && barsToCalculate < firstBarIndexToCalculate) {    // Если не нужно рассчитывать всю..
    firstBarIndexToCalculate = barsToCalculate;                               // ..историю, то начнем с указанного..
  }

  if (calculatedYet == 0) {
    ArrayInitialize(bearishEngulfingBuf, EMPTY_VALUE);
    ArrayInitialize(bullishEngulfingBuf, EMPTY_VALUE);
    return (firstBarIndexToCalculate - 1);
  }
  return 2;
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                        Custom indicator iteration function                                                                     |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]) {
//---
  int limit = recalcIndex(prev_calculated);
  checkingPatterns(limit);
//--- return value of prev_calculated for next call
  return rates_total;
}
  